
#include "EPM8722_ROM.h"


void init_ext_eeprom()
{
   output_float(I2C_SCL);
   output_float(I2C_SDA);

}

void write_ext_eeprom(long int address, BYTE data)
{
   short int status;
   i2c_start();
   i2c_write(EEPROM_WRITE);
   i2c_write(address>>8);
   i2c_write(address);
   i2c_write(data);
   i2c_stop();
   i2c_start();
   // Wait for write operation to finish by polling the EEPROM
   status=i2c_write(EEPROM_WRITE);
   while(status==1)
   {
      i2c_start();
      status=i2c_write(EEPROM_WRITE);
   }
}

BYTE read_ext_eeprom(long int address) {
   BYTE data;
   i2c_start();
   i2c_write(EEPROM_WRITE);
   i2c_write(address>>8);
   i2c_write(address);
   i2c_start();
   i2c_write(EEPROM_READ);
   data=i2c_read(NOACK);
   i2c_stop();
   return(data);
}



// Writes up to 256 bytes block of data to ext eeprom
// 27.3us/byte, 3.4us/bit, 36.63KB/s
void write_ext_eeprom_page(long int address, char *data, long num_bytes)
{
    short int status;

    long i;
	// Setup write mode operation
    i2c_start();
    i2c_write(EEPROM_WRITE);

    // Send starting address of data
    i2c_write(address>>8);
    i2c_write(address);

	// Transfer data into EEPROM page buffer
    for(i=0;i<num_bytes;i++){
        i2c_write(*data++);
    }
    i2c_stop();					// Terminate data write mode
    i2c_start();				// Begin page data flash operation

    // Wait for write operation to finish by polling the EEPROM status register
    status=i2c_write(EEPROM_WRITE);
    while(status==1)
    {
       i2c_start();
       status=i2c_write(EEPROM_WRITE);
    }
}


// Reads up to n bytes data from ext eeprom
// Data is returned by reference only
void read_ext_eeprom_sequential(long int address, char *data, long num_bytes)
{
    //BYTE data;
    long i;
    i2c_start();
    i2c_write(EEPROM_WRITE);
    i2c_write(address>>8);
    i2c_write(address);
    i2c_start();
    i2c_write(EEPROM_READ);
	// Read n bytes of data from EEPROM
    for(i=0;i<num_bytes;i++){
        *data++ = i2c_read(ACK);
    }
	// Terminate read mode
    i2c_stop();
}


void test_EEPROM()
{
    int i,j;
    int buf[256];
    char *ptr = buf;
    long addr;
    int32 time_delta, old_time;

    i=32;j=0;
    //DS1307_read_time();
    for(addr=0; addr<256; addr++){
        //write_ext_eeprom(addr, i++);  // single byte write
        *ptr++ = i++;
        if(i==128)i=32;
        put_COM1('.');
        if(++j == 32){
            printf(put_COM1, "\r\n");
            j=0;
        }
        restart_wdt();
    }

    old_time = timer_count;

	write_ext_eeprom_page(0, buf, 256);
    //write_ext_eeprom_page(8, buf, 8); // For AT24C02

    printf(put_COM1, "\r\nElapsed TIme [ms] = %lu\r\n",
                time_delta = timer_count - old_time);

    memset(buf,0x23,256);

    read_ext_eeprom_sequential(0, buf, 256);
//    read_ext_eeprom_sequential(8, buf, 8);


    ptr = &buf[0];    // Restore pointer to start of data array

    j=0;
    for(addr=0; addr<256; addr++){
        put_COM1(*ptr++);
        if(++j == 16){
            printf(put_COM1, "\r\n");
            j=0;
        }
    }


}